TDSet *make_tdset(const Char_t* basedir, Int_t files_per_slave)
{
   // This script creates a TDSet object that can be used to process
   // the files generated with the make_event_trees.C script.
   // Conventions for file names made by that script are assumed.
   //
   // basedir:         location of files local to proof slaves
   // files_per_slave: number of files per slave to process

   if (!gProof) {
      cout << "Must Start PROOF before using make_tdset.C" << endl;
      return 0;
   }

   if (!basedir) {
      cout << "'basedir' must not be empty" << endl;
      return 0;
   }

   if (files_per_slave <= 0) {
      cout << "files_per_slave must be > 0" << endl;
      return kFALSE;
   }

   THashList nodelist;
   TArrayI nslaves;
   nodelist.SetOwner();
   TList* l = gProof->GetSlaveInfo();
   for(Int_t i=0 ; i < l->GetSize() ; i++){
      TSlaveInfo* si = dynamic_cast<TSlaveInfo*>(l->At(i));
      if (si->fStatus != TSlaveInfo::kActive) continue;
      TObjString* host = 0;
      if(host = dynamic_cast<TObjString*>(nodelist.FindObject(si->fHostName.Data()))) {
         Int_t index = nodelist.IndexOf(host);
         nslaves[index]++;
      } else {
         nodelist.Add(new TObjString(si->fHostName.Data()));
         nslaves.Set(1+nslaves.GetSize());
         nslaves[nslaves.GetSize()-1] = 1;
      }
   }

   TDSet *d = new TDSet("TTree","EventTree");
   for(Int_t i=0; i < nodelist.GetSize() ; i++){
      TObjString* node = dynamic_cast<TObjString*>(nodelist.At(i));
      for(Int_t j=1; j <= files_per_slave*nslaves[i]; j++) {

         TString filestr = "root://";
         filestr += node->GetName();
         filestr += "/";
         filestr += basedir;
         filestr += "/event_tree_";
         filestr += node->GetName();
         filestr += "_";
         filestr += j;
         filestr += ".root";
         d->Add(filestr);
      }
   }

   return d;
}
